/***************************************************************************
  CDropTarget.cpp

  Copyright (C) 2001 Hachimaki. All rights reserved.
  Licensed under the GPL(v2).
***************************************************************************/

#include "CDropTarget.h"

CDropTarget::CDropTarget(UINT cf_idlist, HWND _parent)
{
	m_refs = 1; 
	m_bAcceptFmt = FALSE;
	hwnd_parent = _parent;
	CF_IDLIST = cf_idlist;
	
    fmtetc[0].cfFormat = CF_TEXT;
    fmtetc[0].ptd      = NULL;
    fmtetc[0].dwAspect = DVASPECT_CONTENT;  
    fmtetc[0].lindex   = -1;
    fmtetc[0].tymed    = TYMED_HGLOBAL; 
	
    fmtetc[1].cfFormat = CF_IDLIST;
    fmtetc[1].ptd = NULL;
    fmtetc[1].dwAspect = DVASPECT_CONTENT;
    fmtetc[1].lindex = -1;
    fmtetc[1].tymed = TYMED_HGLOBAL; 
}   

//---------------------------------------------------------------------
//                    IUnknown Methods
//---------------------------------------------------------------------


STDMETHODIMP
CDropTarget::QueryInterface(REFIID iid, void FAR* FAR* ppv) 
{
    if(iid == IID_IUnknown || iid == IID_IDropTarget)
    {
		*ppv = this;
		AddRef();
		return NOERROR;
    }
    *ppv = NULL;
    return ResultFromScode(E_NOINTERFACE);
}


STDMETHODIMP_(ULONG)
CDropTarget::AddRef(void)
{
    return ++m_refs;
}


STDMETHODIMP_(ULONG)
CDropTarget::Release(void)
{
    if(--m_refs == 0)
    {
		delete this;
		return 0;
    }
    return m_refs;
}  

//---------------------------------------------------------------------
//                    IDropTarget Methods
//---------------------------------------------------------------------  

STDMETHODIMP
CDropTarget::DragEnter(LPDATAOBJECT pDataObj, DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect)
{  
	int i;
	
    //t\ȃtH[}bg`FbN
	m_bAcceptFmt = FALSE;
	for (i = 0; i < sizeof(fmtetc) / sizeof(FORMATETC); i++)
	{
		if (NOERROR == pDataObj->QueryGetData(&fmtetc[i]))
		{
			m_bAcceptFmt = TRUE;
			break;
		}
	}
	
    QueryDrop(grfKeyState, pdwEffect);
    return NOERROR;
}

STDMETHODIMP
CDropTarget::DragOver(DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect)
{
    QueryDrop(grfKeyState, pdwEffect);
    return NOERROR;
}

STDMETHODIMP
CDropTarget::DragLeave()
{   
    m_bAcceptFmt = FALSE;   
    return NOERROR;
}

STDMETHODIMP
CDropTarget::Drop(LPDATAOBJECT pDataObj, DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect)  
{   
    STGMEDIUM medium;   
    HGLOBAL hText;
    LPSTR pszText;
	
    if (QueryDrop(grfKeyState, pdwEffect))
    {
		if (SUCCEEDED(pDataObj->GetData(&fmtetc[0], &medium)))
		{
			//CF_TEXT
			hText = medium.hGlobal;
			pszText = (LPSTR)GlobalLock(hText);
			PostMessage(hwnd_parent, WM_USER + 1, 0, 0);
			for (int i = 0; i < (signed)strlen(pszText); i++)
				PostMessage(hwnd_parent, WM_USER + 2, 0, *(pszText + i));
			PostMessage(hwnd_parent, WM_USER + 3, 0, 0);
			GlobalUnlock(hText);
			ReleaseStgMedium(&medium);
			return NOERROR;
		}
		else if (SUCCEEDED(pDataObj->GetData(&fmtetc[1], &medium)))
		{
			//CF_IDLIST
			MessageBox(hwnd_parent, "", NULL, MB_OK);
		}
		else
		{
			*pdwEffect = DROPEFFECT_NONE;
			return E_INVALIDARG; 
		}
    }
	return S_OK;
}   

/* OleStdGetDropEffect
** -------------------
**
** Convert a keyboard state into a DROPEFFECT.
**
** returns the DROPEFFECT value derived from the key state.
**    the following is the standard interpretation:
**          no modifier -- Default Drop     (0 is returned)
**          CTRL        -- DROPEFFECT_COPY
**          SHIFT       -- DROPEFFECT_MOVE
**          CTRL-SHIFT  -- DROPEFFECT_LINK
**
**    Default Drop: this depends on the type of the target application.
**    this is re-interpretable by each target application. a typical
**    interpretation is if the drag is local to the same document
**    (which is source of the drag) then a MOVE operation is
**    performed. if the drag is not local, then a COPY operation is
**    performed.
*/
#define OleStdGetDropEffect(grfKeyState)    \
    ( (grfKeyState & MK_CONTROL) ?          \
	( (grfKeyState & MK_SHIFT) ? DROPEFFECT_LINK : DROPEFFECT_COPY ) :  \
( (grfKeyState & MK_SHIFT) ? DROPEFFECT_MOVE : 0 ) )

//---------------------------------------------------------------------
// CDropTarget::QueryDrop: Given key state, determines the type of 
// acceptable drag and returns the a dwEffect. 
//---------------------------------------------------------------------   
STDMETHODIMP_(BOOL)
CDropTarget::QueryDrop(DWORD grfKeyState, LPDWORD pdwEffect)
{  
    DWORD dwOKEffects = *pdwEffect; 
    
    if (!m_bAcceptFmt)
        goto dropeffect_none; 
	
    *pdwEffect = OleStdGetDropEffect(grfKeyState);
    if (*pdwEffect == 0) {
        // No modifier keys used by user while dragging. Try in order: MOVE, COPY.
        if (DROPEFFECT_MOVE & dwOKEffects)
            *pdwEffect = DROPEFFECT_MOVE;
        else if (DROPEFFECT_COPY & dwOKEffects)
            *pdwEffect = DROPEFFECT_COPY; 
        else goto dropeffect_none;   
    } 
    else {
        // Check if the drag source application allows the drop effect desired by user.
        // The drag source specifies this in DoDragDrop
        if (!(*pdwEffect & dwOKEffects))
            goto dropeffect_none; 
        // We don't accept links
        if (*pdwEffect == DROPEFFECT_LINK)
            goto dropeffect_none; 
    }  
    return TRUE;
	
dropeffect_none:
    *pdwEffect = DROPEFFECT_NONE;
    return FALSE;
}   
