/*****************************************************************************
  Gicko.h

  Copyright (C) 2001 Hachimaki. All rights reserved.
  Licensed under the GPL(v2).
*****************************************************************************/

#ifndef __GICKO_H__
#define __GICKO_H__

#pragma warning(disable : 4786)

#include <windows.h>
#include <wininet.h>
#include <stdio.h>
#include <time.h>
#include <algorithm>
#include <functional>
#include <vector>
#include <string>

#ifndef _DEBUG
#define AGENT               "Gicko - 2ch browse engine"
#else
#define AGENT				"Mozilla/4.75 [ja] (WinNT; U)"
#endif
#define HEADER              "Accept: */*\r\n"\
                            "Accept-Encoding: gzip, deflate\r\n"\
							"\r\n"
//#define URL_BBSTABLE		"http://www6.ocn.ne.jp/~mirv/bbstable.html"
//#define URL_BBSTABLE		"http://www.2ch.net/newbbsmenu.html"
// 2002/02/12 modified
//#define URL_BBSTABLE		"http://www.2ch.net/2chmenu.html"
#define URL_BBSTABLE		"http://www6.ocn.ne.jp/~mirv/2chmenu.html"
#define OVER_1000_THREAD	"Over 1000 Thread"
#define SONNA_THREAD_NAI	"ȔorXbhȂłB"

#define MAX_ITA_NAME        33
#define MAX_CATG_NAME       33
#define MAX_SERVER_NAME     MAX_PATH
#define MAX_BBS_NAME        33
#define MAX_THREAD_KEY      15
#define MAX_FROM_NAME		67
#define MAX_ID				65
#define MAX_ERROR_MESSAGE	256

#define CHECKINDEX(i,r)		if ((i) < 0 || ((unsigned)i) >= items.size()) \
							{ wsprintf(error_message, "ʒu %d ͖", i); \
							return (r); }

#define REWINDRET(p,r)		{ fseek(fp, (p), SEEK_SET); return (r); }

using namespace std;


/*****************************************************************************
  class GUrl
*****************************************************************************/

class GUrl
{
private:
	bool split_pathmode(const char* url);
public:
	char server[MAX_SERVER_NAME];
	char bbs[MAX_BBS_NAME];
	char key[MAX_THREAD_KEY];
	unsigned int ls;
	unsigned int st;
	unsigned int to;
	bool nofirst;
	UCHAR type;
	bool split(const char* url);
	void make(char* url);
};


/*****************************************************************************
  class GBase
*****************************************************************************/

class GBase
{

protected:

	HWND hwnd_callback;
	UINT msg_callback;
	LPARAM data_callback;

	char* get_html(const char* url, size_t* buffsize);
	void inet_callback(DWORD status);

public:

	GBase(void)
	{
		hwnd_callback = NULL;
		msg_callback = NULL;
	}

	void set_callback(HWND _hwnd, UINT _msg, LPARAM _data)
	{
		hwnd_callback = _hwnd;
		msg_callback = _msg;
		data_callback = _data;
	}

	static void CALLBACK _inet_callback(
		HINTERNET hInternet,
		DWORD dwContext,
		DWORD dwInternetStatus,
		LPVOID lpvStatusInformation,
		DWORD dwStatusInformationLength)
	{
		((GBase*)dwContext)->inet_callback(dwInternetStatus);
	}

	bool is_2ch(const char* url)
	{
		if (strstr(url, ".2ch.net") || strstr(url, ".bbspink.com"))
			return true;
		return false;
	}

	void _get_log_filename(char* logfile, const char* log_dir, const char* _s, const char* _b, const char* _k, UCHAR _ty)
	{
		char* p;

		if (log_dir != NULL)
			sprintf(logfile, "%s\\", log_dir);
		else
			strcpy(logfile, "");

		switch (_ty)
		{
		case 0:
			sprintf(logfile, "%s%s%s.dat", logfile, _b, _k);
			break;
		case 1:
			sprintf(logfile, "%s%s_%s_%s", logfile, _s, _b, _k);
			while ((p = strchr(logfile, '.')) != NULL) *p = '_';
			while ((p = strchr(logfile, '/')) != NULL) *p = '_';
			strcat(logfile, ".dat");
			break;
		case 255:
			sprintf(logfile, "%s%s", logfile, _s);
			break;
		}
	}
	
	void convert_er(string &str);
};


/*****************************************************************************
  class GItaList
*****************************************************************************/

class GItaList : public GBase
{

protected:

	class ItaItem
	{
	public:
		char category[MAX_CATG_NAME];
		char name[MAX_ITA_NAME];
		char server[MAX_SERVER_NAME];
		char bbs[MAX_BBS_NAME];
		UCHAR type;
		long param;
		ItaItem(void)
		{
			memset(name,     0, MAX_ITA_NAME);
			memset(category, 0, MAX_CATG_NAME);
			memset(server,   0, MAX_SERVER_NAME);
			memset(bbs,      0, MAX_BBS_NAME);
			type = 0;
			param = 0;
		}
	};

	class pred_itamei_equal
		: public binary_function<char*, ItaItem, bool>
	{
	public:
		bool operator()(const char* name, const ItaItem &item) const
		{
			return (strcmp(name, item.name) == 0);
		}
	};

    vector<ItaItem> items;
	char filename[MAX_PATH];
    char error_message[MAX_ERROR_MESSAGE];

public:

	GItaList(void);
    bool get(const char* url);
	bool load(void);
	bool save(void);
	void set_filename(const char* _filename);

	int         get_no(const char* name);
    const char* get_name(int index);
    const char* get_category(int index);
    const char* get_category(const char* name);
    const char* get_server(int index);
    const char* get_server(const char* name);
    const char* get_bbs(int index);
    const char* get_bbs(const char* name);
	UCHAR       get_type(int index);
	UCHAR       get_type(const char* name);
    bool get_topurl(int index, char* url);
    bool get_topurl(const char* name, char* url);
    UINT count(void);

	bool set_param(int index, long param);
	bool set_param(const char* name, long param);
	long get_param(int index);
	long get_param(const char* name);

	void add(const char* n, const char* c, const char* s, const char* b, long p);
	bool modify(int index, const char* n, const char* c, const char* s, const char* b);
	bool del(int index);
	void clear(void);

    const char* get_error(void);
};


/*****************************************************************************
  class GSureList
*****************************************************************************/

class GSureList : public GBase
{

protected:

	class Item
	{
	public:
	    char   server[MAX_SERVER_NAME];
	    char   bbs[MAX_BBS_NAME];
		char   key[MAX_THREAD_KEY];
		char   from[MAX_FROM_NAME];
		string title;
		UINT   res;
		UCHAR  type;
		UCHAR  flag;
	};
    vector<Item> items;

    char error_message[MAX_ERROR_MESSAGE];

protected:

	bool get_from_subject_txt(char* buff, size_t size, UCHAR type);

public:

	GSureList(void);
	bool get(const char* _s, const char* _b, const char* _f);
	void insert(UINT i, const char* _s, const char* _b, const char* _k, const char* _t, UINT _r, UCHAR _ty, const char* _f);
	bool modify(UINT i, const char* _t, const char* _f);
	bool del(int i);

    UINT		count(void)				{ return items.size(); }
    const char* get_server(int i)		{ CHECKINDEX(i,NULL); return items[i].server; }
    const char* get_bbs(int i)			{ CHECKINDEX(i,NULL); return items[i].bbs; }
    const char* get_key(int i)			{ CHECKINDEX(i,NULL); return items[i].key; }
	const char* get_from(int i)			{ CHECKINDEX(i,NULL); return items[i].from; }
    const char* get_title(int i)		{ CHECKINDEX(i,NULL); return items[i].title.c_str(); }
    UINT		get_res(int i)			{ CHECKINDEX(i,0); return items[i].res; }
	UCHAR		get_type(int i)			{ CHECKINDEX(i,0); return items[i].type; }
	UCHAR       get_flag(int i)			{ CHECKINDEX(i,0); return items[i].flag; }
	bool        set_flag(int i, UCHAR f){ CHECKINDEX(i,false); items[i].flag = f; return true; }
    bool		get_url(int index, int ls, char* url);
	bool		get_log_filename(char* logfile, const char* log_dir, int i);

	const char* get_error(void)			{ return error_message; }
};


/*****************************************************************************
  class GSure
*****************************************************************************/

class GSure : public GBase
{

protected:

	class Res
	{
	public:
		string name;
		string mail;
		time_t wtime;
		char id[MAX_ID];
		vector<string> text;
	};
	vector<Res> res;

	string title;
    char server[MAX_SERVER_NAME];
    char bbs[MAX_BBS_NAME];
	char key[MAX_THREAD_KEY];
	UCHAR type;
	char cgiver[6];
	char error_message[256];

protected:

	bool get_title_from_html(char** mp, char* title);
	UINT get_res_from_html(char** mp, Res& re);
	bool get_title_from_log(FILE* fp, char* title);
	UINT get_res_from_log(FILE* fp, Res& re);
	bool load_log(const char* log_dir);
	bool save_log(const char* log_dir);

public:

	GSure(void);
	bool  get(const char* url, const char* log_dir, int* st, bool offline);

	const char* get_server(void)		{ return server; }
	const char* get_bbs(void)			{ return bbs; }
	const char* get_key(void)			{ return key; }
	const char* get_cgiver(void)		{ return cgiver; }
	const char* get_title(void)			{ return title.c_str(); }
	UCHAR       get_type(void)          { return type; }

	const char* get_name(int i)			{ return res[i].name.c_str(); }
	const char* get_mail(int i)			{ return res[i].mail.c_str(); }
	time_t      get_wtime(int i)		{ return res[i].wtime; }
	const char* get_id(int i)			{ return res[i].id; }
	const char* get_text(int i, int j)	{ return res[i].text[j].c_str(); }
	UINT        textcount(int i)		{ return res[i].text.size(); }
	UINT        count(void)				{ return res.size(); }
    bool		get_url(char* url);
	void		set_cgiver(const char* _c) { strcpy(cgiver, _c); }

    const char* get_error(void)			{ return error_message; }
};


#endif
