/***************************************************************************
  GickoViewer.h

  Copyright (C) 2001 Hachimaki. All rights reserved.
  Licensed under the GPL(v2).
***************************************************************************/

#ifndef __GICKOVIEWER_H__
#define __GICKOVIEWER_H__

#include "Gicko.h"

#ifndef IDC_HAND
#define IDC_HAND						MAKEINTRESOURCE(32649)
#endif

#define IDM_COPY						1
#define IDM_FIND						2
#define IDM_OPENBYBROWSER				3
#define IDM_ADDFAVORITE					4
#define IDM_WRITE						5
#define IDM_READCGIVER					6
#define IDM_URL_COPY					7
#define IDM_URL_OPENBYBROWSER			8
#define IDM_URL_CHECK					9

#define GVIEWER_CALLBACK_KEYDOWN		100
#define GVIEWER_CALLBACK_VSCROLL		101
#define GVIEWER_CALLBACK_ADDFAVORITE	102
#define GVIEWER_CALLBACK_OPENBYBROWSER	103
#define GVIEWER_CALLBACK_ONHYPERLINK	104
#define GVIEWER_CALLBACK_CLICKHYPERLINK	105
#define GVIEWER_CALLBACK_POSTMESSAGE	106
#define GVIEWER_CALLBACK_OPENFIND		107

#define BCCHECKER_URL					"http://www.jah.ne.jp/~fild/cgi-bin/LBCC/lbcc.cgi?"

/***************************************************************************
  GViewer
***************************************************************************/

class GViewer : public GSure
{

private:

	class LinkArea
	{
	public:
		RECT  r;
		USHORT ui;
	};

	class Line
	{
	public:
		vector<USHORT> st;
		vector<USHORT> ln;
		vector<UCHAR>  si;
		vector<USHORT> ui;
		string str;
		void clear(void)
		{
			st.clear();
			ln.clear();
			si.clear();
			ui.clear();
			str = "";
		};
		void add(const char* _str, UCHAR _si, USHORT _ui = USHRT_MAX)
		{
			str += _str;
			st.push_back(str.size() - strlen(_str));
			ln.push_back(strlen(_str));
			si.push_back(_si);
			ui.push_back(_ui);
		};
	};

	class Style
	{
	public:
		LOGFONT		font;
		COLORREF	color;
		COLORREF	bkcolor;
	};

private:

	vector<Line>			doc;
	vector<string>			docurl;
	vector<LinkArea>		la;
	Style					style[7];

	UINT UM_MOUSEWHEEL;
	HWND hwnd_this;
	HWND hwnd_gtgt;
	UINT callback;
	int gtgt_no;
	string copybuff;
	USHORT onmouse_ui_idx;
	int top_line;

	//drag parameter
	POINT dragstart_pt;
	POINT dragend_pt;
	bool dragging;

public:

	//setting parameter
	LOGFONT font;
	int font_cy;
	int indent;
	char body_bo; char body_it; char body_ul;
	char link_bo; char link_it; char link_ul;
	char numb_bo; char numb_it; char numb_ul;
	char name_bo; char name_it; char name_ul;
	char date_bo; char date_it; char date_ul;
	char idid_bo; char idid_it; char idid_ul;
	char numb_on; 
	char name_on; 
	char date_on; 
	char idid_on; 
	bool mailout;
	COLORREF body_cl;
	COLORREF link_cl;
	COLORREF numb_cl;
	COLORREF name_cl;
	COLORREF date_cl;
	COLORREF idid_cl;
	COLORREF back_cl;
	vector<string> post_name;
	vector<string> post_mail;

private:

	void on_paint(HDC hdc);
	void on_size(UINT type, int x, int y);
	void on_vscroll(UINT code, short int pos, HWND hwnd);
	void on_mousemove(DWORD key, POINT pt);
	void on_lbuttondown(DWORD key, POINT pt);
	void on_lbuttonup(DWORD key, POINT pt);
	void on_keydown(int key, UINT data);
	void on_command(WORD notify, WORD id, HWND hwnd);
	void on_contextmenu(HWND hwnd, WORD x, WORD y);
	void set_vscrollbar(void);

	static BOOL CALLBACK dlg_post_proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
	void gtgtwnd_OnPaint(HDC hdc);
	void gtgtwnd_get_size(SIZE* size);
	bool post_res(const char* name, const char* mail, char* res);
	void urlencode(const char* in, char *out);
	void res_to_doc(int no, Res& re);
	void check_res_pointer(char* tmp);
	void add_hokan(int st, int to);
	void add_kousin(int st);
	int  draw_line(int line, HDC hdc, int y, bool add_la);
	void _makestyle(void);
	int  get_noline(int no);
	void copy_to_clipboard(void);
	void send_callback(WPARAM wp, LPARAM lp);

public:

	GViewer(void);
	static LRESULT CALLBACK proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
	LRESULT viewer_proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
	static LRESULT CALLBACK gtgtwnd_Proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);

	bool create(HWND hwnd_parent, UINT id, UINT callback);
	bool open(const char* url, const char* log_dir, bool direct, UINT _top_line);
	void show(bool flag);
	void set_pos(int x, int y);
	void set_size(int w, int h);
	bool findstring(FINDREPLACE* pfr);

	void set_font(const LOGFONT* newlf);
	void set_body_flag(char bo, char it, char ul);
	void set_link_flag(char bo, char it, char ul);
	void set_numb_flag(char on, char bo, char it, char ul);
	void set_name_flag(char on, char bo, char it, char ul);
	void set_date_flag(char on, char bo, char it, char ul);
	void set_idid_flag(char on, char bo, char it, char ul);
	void set_color(COLORREF bo, COLORREF li, COLORREF nu,
		           COLORREF na, COLORREF da, COLORREF id,
				   COLORREF bk);

	HWND get_handle(void) { return hwnd_this; };
	UINT get_topline(void) { return top_line; };

};


#endif
