/***************************************************************************
  SureList.cpp

  Copyright (C) 2001 Hachimaki. All rights reserved.
  Licensed under the GPL(v2).
***************************************************************************/

#include "GBCommon.h"
#include <process.h>

class ExportDat : public GSure
{
public:
	bool output(const char* infile, const char* outfile)
	{
		Res re;
		FILE* in;
		FILE* out;
		char _title[256];
		struct tm* ptm;
		bool first = true;
		UINT i;
		if ((in = fopen(infile, "r")) == NULL)
		{
			return false;
		}
		if ((out = fopen(outfile, "w")) == NULL)
		{
			fclose(in);
			return false;
		}
		if (!get_title_from_log(in, _title))
		{
			fclose(in);
			fclose(out);
			return false;
		}
		while (get_res_from_log(in, re) != UINT_MAX)
		{
			fprintf(out, "%s<>%s<>", re.name.c_str(), re.mail.c_str());
			ptm = localtime(&re.wtime);
			fprintf(out, " %02d/%02d/%02d %02d:%02d (%s)",
				ptm->tm_year - 100,
				ptm->tm_mon + 1,
				ptm->tm_mday,
				ptm->tm_hour,
				ptm->tm_min,
				"\0\0\0\0\0\0y\0" + ptm->tm_wday * 3);
			if (strlen(re.id) != 0)
			{
				fprintf(out, " ID:%s", re.id);
			}
			fprintf(out, "<>");
			for (i = 0; i < re.text.size() - 1; i++)
			{
				if (i == 0)
					fprintf(out, "%s", re.text[i]);
				else
					fprintf(out, "<br>%s", re.text[i]);
			}
			if (first)
			{
				fprintf(out, "<>%s\n", _title);
				first = false;
			}
			else
				fprintf(out, "<>\n");

		}
		fclose(in);
		fclose(out);
		return true;
	}
};

extern HINSTANCE instance;
extern HWND hwnd_main;
extern HWND hwnd_list;
extern HANDLE handle_get_thread;
extern HANDLE handle_check_thread;
extern UINT timer_id;
extern GSureList* surelist_def;
extern GSureList* surelist_fav;
extern GSureList* surelist_rec;
extern char log_dir[MAX_PATH];
extern char image_file[MAX_PATH];
extern int list_type;
extern BOOL tab_automove_check;

static void set_list_icon(int item);
static LPARAM get_list_param(int item);
static int CALLBACK list_compare(LPARAM lp1, LPARAM lp2, LPARAM lp3);
static UINT WINAPI threadentry_check_thread(void* data);
BOOL CALLBACK dlg_sureprop_Proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
void dlg_sureprop_OnInit(HWND hwnd, GSureList* surelist, UINT idx);


/*--------------------------------------------------------------------------

  create_surelist

  XXgr[쐬

--------------------------------------------------------------------------*/

void create_surelist(void)
{
	LVCOLUMN col;

	hwnd_list = CreateWindowEx(
		WS_EX_CLIENTEDGE,
		WC_LISTVIEW,
		"",
		WS_CHILD | WS_CLIPSIBLINGS | WS_TABSTOP |
		LVS_REPORT | LVS_SINGLESEL | LVS_SHOWSELALWAYS,
		0, 0, 0, 0,
		hwnd_main,
		(HMENU)ID_LIST,
		instance,
		NULL);

	ListView_SetExtendedListViewStyle(hwnd_list, LVS_EX_FULLROWSELECT);

	//Jݒ
	col.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
	col.fmt = LVCFMT_LEFT;
	col.cx = 40;
	col.pszText = "";
	col.iSubItem = 0;
	ListView_InsertColumn(hwnd_list, 0, &col);

	col.fmt = LVCFMT_LEFT;
	col.cx = 320;
	col.pszText = "薼";
	col.iSubItem = 1;
	ListView_InsertColumn(hwnd_list, 1, &col);

	col.fmt = LVCFMT_RIGHT;
	col.cx = 40;
	col.pszText = "X";
	col.iSubItem = 2;
	ListView_InsertColumn(hwnd_list, 2, &col);

	col.fmt = LVCFMT_LEFT;
	col.cx = 100;
	col.pszText = "ꏊ";
	col.iSubItem = 3;
	ListView_InsertColumn(hwnd_list, 3, &col);
}


/*--------------------------------------------------------------------------
 
  set_surelist_image

  C[WXgZbg

--------------------------------------------------------------------------*/

void set_surelist_image(void)
{
	HIMAGELIST imagelist;
	HBITMAP bitmap;

	if (strlen(image_file) != 0)
	{
		if ((imagelist = create_imagelist()) != NULL)
		{
			ListView_SetImageList(hwnd_list, imagelist, LVSIL_SMALL);
			return;
		}
	}

	imagelist = ImageList_Create(16, 16, ILC_COLOR8 | ILC_MASK, 10, 0);
	ImageList_SetBkColor(imagelist, GetSysColor(COLOR_WINDOW));
	bitmap = LoadBitmap(instance, MAKEINTRESOURCE(IDB_IMAGELIST));
	ImageList_AddMasked(imagelist, bitmap, RGB(0xFF, 0x00, 0xFF));
	DeleteObject(bitmap);

	ListView_SetImageList(hwnd_list, imagelist, LVSIL_SMALL);
}


/*--------------------------------------------------------------------------

  surelist_OnColumnClick

  JNbN̏

--------------------------------------------------------------------------*/

void surelist_OnColumnClick(int iSubItem)
{
	static int sortkey;
	static int sortdir;

	if (sortkey == iSubItem)
	{
		if (sortdir == 2)
			sortdir = 1;
		else
			sortdir = 2;
	}
	else
	{
		sortkey = iSubItem;
		sortdir = 2;
	}
	ListView_SortItems(hwnd_list, list_compare, MAKELPARAM(sortdir, sortkey));
}


/*--------------------------------------------------------------------------
 
  surelist_OnContextMenu

  ENbNj[\

--------------------------------------------------------------------------*/

void surelist_OnContextMenu(WORD x, WORD y)
{
	LVHITTESTINFO list_ht;
	MENUITEMINFO mii;
	HMENU menu1;
	HMENU menu2;
	RECT r;
	int item;

	if (x == USHRT_MAX && y == USHRT_MAX)
	{
		item = ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED);
		ListView_GetItemRect(hwnd_list, item, &r, LVIR_BOUNDS);
		MapWindowPoints(hwnd_list, HWND_DESKTOP, (POINT*)&r, 2);
		x = (USHORT)r.left;
		y = (USHORT)r.bottom;
	}
	else
	{
		//}EX̃ACe擾
		list_ht.pt.x = x;
		list_ht.pt.y = y;
		ScreenToClient(hwnd_list, &list_ht.pt);
		item = ListView_HitTest(hwnd_list, &list_ht);
	}

	//j[C
	if (item != -1)
	{
		switch (list_type)
		{
		case 0: menu1 = LoadMenu(instance, MAKEINTRESOURCE(IDR_THREAD_POPUP_DEFAULT)); break;
		case 1: menu1 = LoadMenu(instance, MAKEINTRESOURCE(IDR_THREAD_POPUP_FAVORITE)); break;
		case 2: menu1 = LoadMenu(instance, MAKEINTRESOURCE(IDR_THREAD_POPUP_RECENT)); break;
		}
	}
	else
	{
		if (list_type == 1)
			menu1 = LoadMenu(instance, MAKEINTRESOURCE(IDR_THREAD_POPUP_NONE));
		else
			return;
	}
	
	menu2 = GetSubMenu(menu1, 0);
	if (list_type == 1 && surelist_fav->get_type(LOWORD(get_list_param(item))) == 255)
	{
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_STATE;
		mii.fState = MFS_DISABLED;
		SetMenuItemInfo(menu2, IDM_OPEN_SURE_BY_BROWSER, FALSE, &mii);
		SetMenuItemInfo(menu2, IDM_CHECK_SURE, FALSE, &mii);
		SetMenuItemInfo(menu2, IDM_CHECK_ALL_SURE, FALSE, &mii);
	}
	
	TrackPopupMenu(menu2, TPM_LEFTALIGN | TPM_LEFTBUTTON, x, y, 0, hwnd_main, NULL);
	DestroyMenu(menu1);
}


/*--------------------------------------------------------------------------

  make_surelist

  XXg쐬

--------------------------------------------------------------------------*/

void make_surelist(int type)
{
	LVITEM item;
	char tmp[256];
	UINT i;

	if (type == -1)
		list_type = (list_type + 1) % 3;
	else if (type == -2)
		list_type = (list_type + 2) % 3;
	else
		list_type = type;

	ListView_DeleteAllItems(hwnd_list);

	//Xgɒǉ
	switch (list_type)
	{
	case 0:
		if (handle_get_thread != NULL)
		{
			EnableWindow(hwnd_list, FALSE);
			item.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
			item.pszText = NULL;
			item.iItem = 0;
			item.iSubItem = 0;
			item.iImage = -1;
			item.lParam = -1;
			ListView_InsertItem(hwnd_list, &item);
			
			item.mask = LVIF_TEXT;
			item.pszText = "Xbh擾...";
			item.iItem = 0;
			item.iSubItem = 1;
			ListView_SetItem(hwnd_list, &item);
			wsprintf(tmp, "X");
		}
		else
		{
			EnableWindow(hwnd_list, TRUE);
			for (i = 0; i < surelist_def->count(); i++)
			{
				wsprintf(tmp, "%d", i + 1);
				item.mask = LVIF_TEXT | LVIF_PARAM;
				item.pszText = tmp;
				item.iItem = i;
				item.iSubItem = 0;
				item.lParam = MAKELPARAM(i, 0);
				ListView_InsertItem(hwnd_list, &item);

				strcpy(tmp, surelist_def->get_title(i));
				item.mask = LVIF_TEXT;
				item.pszText = tmp;
				item.iItem = i;
				item.iSubItem = 1;
				ListView_SetItem(hwnd_list, &item);
					
				sprintf(tmp, "%d", surelist_def->get_res(i));
				item.pszText = tmp;
				item.iItem = i;
				item.iSubItem = 2;
				ListView_SetItem(hwnd_list, &item);

				strcpy(tmp, surelist_def->get_from(i));
				item.pszText = tmp;
				item.iItem = i;
				item.iSubItem = 3;
				ListView_SetItem(hwnd_list, &item);

				set_list_icon(i);
			}
			if (surelist_def->count() > 0)
				wsprintf(tmp, "X[%s]", surelist_def->get_from(0));
			else
				wsprintf(tmp, "X");
		}
		break;

	case 1:
		EnableWindow(hwnd_list, TRUE);
		for (i = 0; i < surelist_fav->count(); i++)
		{
			item.mask = LVIF_TEXT | LVIF_PARAM;
			item.pszText = "";
			item.iItem = i;
			item.iSubItem = 0;
			item.lParam = MAKELPARAM(i, 1);
			ListView_InsertItem(hwnd_list, &item);

			strcpy(tmp, surelist_fav->get_title(i));
			item.mask = LVIF_TEXT;
			item.pszText = tmp;
			item.iItem = i;
			item.iSubItem = 1;
			ListView_SetItem(hwnd_list, &item);

			item.pszText = "";
			item.iItem = i;
			item.iSubItem = 2;
			ListView_SetItem(hwnd_list, &item);

			strcpy(tmp, surelist_fav->get_from(i));
			item.pszText = tmp;
			item.iItem = i;
			item.iSubItem = 3;
			ListView_SetItem(hwnd_list, &item);

			set_list_icon(i);
		}
		wsprintf(tmp, "X[Cɓ]");
		break;

	case 2:
		EnableWindow(hwnd_list, TRUE);
		for (i = 0; i < surelist_rec->count(); i++)
		{
			item.mask = LVIF_TEXT | LVIF_PARAM;
			item.pszText = "";
			item.iItem = i;
			item.iSubItem = 0;
			item.lParam = MAKELPARAM(i, 2);
			ListView_InsertItem(hwnd_list, &item);

			strcpy(tmp, surelist_rec->get_title(i));
			item.mask = LVIF_TEXT;
			item.pszText = tmp;
			item.iItem = i;
			item.iSubItem = 1;
			ListView_SetItem(hwnd_list, &item);

			item.pszText = "";
			item.iItem = i;
			item.iSubItem = 2;
			ListView_SetItem(hwnd_list, &item);

			strcpy(tmp, surelist_rec->get_from(i));
			item.pszText = tmp;
			item.iItem = i;
			item.iSubItem = 3;
			ListView_SetItem(hwnd_list, &item);

			set_list_icon(i);
		}
		wsprintf(tmp, "X[ŋ]");
		break;
	}

	if (list_type != 0 || handle_get_thread == NULL)
		ListView_SetItemState(hwnd_list, 0, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	set_tab_text(1, tmp);
}


/*--------------------------------------------------------------------------
 
  get_list_type

  Xg^CvԂ

--------------------------------------------------------------------------*/

int get_list_type(void)
{
	return list_type;
}


/*--------------------------------------------------------------------------
 
  add_favorite_sure

  XCɓɒǉ

--------------------------------------------------------------------------*/

void add_favorite_sure(void)
{
	LPARAM param;
	int idx;

	param = get_list_param(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED));
	idx = LOWORD(param);

	switch (HIWORD(param))
	{
	case 1:
		return;
	case 0:
		surelist_fav->insert(
			surelist_fav->count(),
			surelist_def->get_server(idx),
			surelist_def->get_bbs(idx),
			surelist_def->get_key(idx),
			surelist_def->get_title(idx),
			0,
			surelist_def->get_type(idx),
			surelist_def->get_from(idx));
		break;
	case 2:
		surelist_fav->insert(
			surelist_fav->count(),
			surelist_rec->get_server(idx),
			surelist_rec->get_bbs(idx),
			surelist_rec->get_key(idx),
			surelist_rec->get_title(idx),
			0,
			surelist_rec->get_type(idx),
			surelist_rec->get_from(idx));
		break;
	}

	save_thread();
}


/*--------------------------------------------------------------------------
 
  del_favorite_sure

  Cɓ̃X폜

--------------------------------------------------------------------------*/

void del_favorite_sure(void)
{
	LPARAM param;
	int idx;

	param = get_list_param(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED));
	idx = LOWORD(param);

	if (HIWORD(param) == 1)
	{
		surelist_fav->del(idx);
		save_thread();
		make_surelist(1);
	}
}


/*--------------------------------------------------------------------------
 
  open_sure

  XJ

--------------------------------------------------------------------------*/

void open_sure(void)
{
	GSureList* surelist;
	LPARAM param;
	char url[256];
	int idx;
	UINT i;

	param = get_list_param(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED));
	idx = LOWORD(param);

	switch (HIWORD(param))
	{
	case 0: surelist = surelist_def; break;
	case 1: surelist = surelist_fav; break;
	case 2: surelist = surelist_rec; break;
	}

	//ŋߊJXɒǉ
	if (HIWORD(param) != 2 && surelist->get_type(idx) != 255)
	{
		for (i = 0; i < surelist_rec->count(); i++)
		{
			if (strcmp(surelist->get_server(idx), surelist_rec->get_server(i)) == 0 &&
				strcmp(surelist->get_bbs(idx),    surelist_rec->get_bbs(i))    == 0 &&
				strcmp(surelist->get_key(idx),    surelist_rec->get_key(i))    == 0)
			{
				surelist_rec->del(i);
				break;
			}
		}
		surelist_rec->insert(
			0,
			surelist->get_server(idx),
			surelist->get_bbs(idx),
			surelist->get_key(idx),
			surelist->get_title(idx),
			0,
			surelist->get_type(idx),
			surelist->get_from(idx));
		save_thread();
	}

	if (surelist->get_flag(idx) == 3)
	{
		surelist->set_flag(idx, 0);
		set_list_icon(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED));
	}

	surelist->get_url(idx, 0, url);
	open_tab(url, surelist->get_type(idx), surelist->get_title(idx));
}


/*--------------------------------------------------------------------------
 
  open_sure_by_browser

  XuEUŊJ

--------------------------------------------------------------------------*/

void open_sure_by_browser(void)
{
	GSureList* surelist;
	LPARAM param;
	char cmdline[1024];
	int idx;

	param = get_list_param(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED));
	idx = LOWORD(param);
	strcpy(cmdline, "exec.exe ");

	switch (HIWORD(param))
	{
	case 0: surelist = surelist_def; break;
	case 1: surelist = surelist_fav; break;
	case 2: surelist = surelist_rec; break;
	}
	surelist->get_url(idx, 0, cmdline + 9);
	WinExec(cmdline, SW_SHOWNORMAL);
}


/*--------------------------------------------------------------------------
 
  check_sure

  X̍XV`FbN

--------------------------------------------------------------------------*/

void check_sure(bool all)
{
	unsigned int taddr;
	int idx;
	int i;

	if (all)
	{
		for (i = 0; i < ListView_GetItemCount(hwnd_list); i++)
		{
			idx = LOWORD(get_list_param(i));
			if (surelist_fav->get_type(idx) != 255)
			{
				surelist_fav->set_flag(idx, 1);
				set_list_icon(i);
			}
		}
	}
	else
	{
		idx = LOWORD(get_list_param(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED)));
		surelist_fav->set_flag(idx, 1);
		set_list_icon(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED));
	}

	if (handle_check_thread == NULL)
		handle_check_thread = (HANDLE)_beginthreadex(NULL, 0,
										  threadentry_check_thread, 0, 0, &taddr);
}


/*--------------------------------------------------------------------------
 
  find_sure

  X

--------------------------------------------------------------------------*/

void find_sure(const char* keyword)
{
    char title[256];
    char* p;
    bool found;
    int cnt;
    int si;
    int i;

	cnt = ListView_GetItemCount(hwnd_list);
	si = ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED);
	
	i = si;
	found = false;
	while (!found)
	{
		i++;
		if (i == cnt)
			i = 0;
		if (i == si)
			break;
		
		ListView_GetItemText(hwnd_list, i, 1, title, sizeof(title));
		for (p = title; *p != '\0'; p++)
		{
			if (_strnicmp(p, keyword, strlen(keyword)) == 0)
			{
				ListView_SetItemState(
					hwnd_list,
					i,
					LVIS_SELECTED | LVIS_FOCUSED,
					LVIS_SELECTED | LVIS_FOCUSED);
				found = true;
			}
		}
	}
	
	if (found)
	{
		RECT ir, cr;
		
		GetClientRect(hwnd_list, &cr);
		ListView_GetItemRect(hwnd_list, i, &ir, LVIR_BOUNDS);
		
		if (ir.top - (ir.bottom - ir.top) < cr.top) /*wb_ČvZ*/
			ListView_Scroll(hwnd_list, 0, ir.top - (ir.bottom - ir.top));
		else if (ir.bottom > cr.bottom)
			ListView_Scroll(hwnd_list, 0, ir.bottom - cr.bottom);
	}
}


/*--------------------------------------------------------------------------
 
  show_sure_property

  XEvpeB\

--------------------------------------------------------------------------*/

void show_sure_property(void)
{
	if (DialogBox(instance, MAKEINTRESOURCE(IDD_SURE_PROPERTY), hwnd_list, dlg_sureprop_Proc) == IDOK)
		make_surelist(list_type);
}


/*--------------------------------------------------------------------------
 
  set_list_icon

  XgACẽACRύX

--------------------------------------------------------------------------*/

void set_list_icon(int item)
{
	LPARAM param;
	WIN32_FIND_DATA wfd;
	HANDLE h;
	char logfile[MAX_PATH];
	int iidx;
	int idx;

	param = get_list_param(item);
	idx = LOWORD(param);

	change_to_moduledir();

	switch (HIWORD(param))
	{
	case 0:
		surelist_def->get_log_filename(logfile, log_dir, idx);
		if ((h = FindFirstFile(logfile, &wfd)) == INVALID_HANDLE_VALUE)
		{
			iidx = 4;
			break;
		}
		iidx = 5;
		FindClose(h);
		break;

	case 1:
		if (surelist_fav->get_type(idx) == 255)
		{
			iidx = 6;
		}
		else
		{
			switch (surelist_fav->get_flag(idx))
			{
			case 0:
				surelist_fav->get_log_filename(logfile, log_dir, idx);
				if ((h = FindFirstFile(logfile, &wfd)) == INVALID_HANDLE_VALUE)
				{
					iidx = 4;
					break;
				}
				iidx = 5;
				FindClose(h);
				break;
			case 1:
				iidx = 7;
				break;
			case 2:
				iidx = 8;
				break;
			case 3:
				iidx = 9;
				break;
			}
		}
		break;

	case 2:
		surelist_rec->get_log_filename(logfile, log_dir, idx);
		if ((h = FindFirstFile(logfile, &wfd)) == INVALID_HANDLE_VALUE)
		{
			iidx = 4;
			break;
		}
		iidx = 5;
		FindClose(h);
		break;
	}

	LVITEM lvi;
	lvi.mask = LVIF_IMAGE;
	lvi.iItem = item;
	lvi.iSubItem = 0;
	lvi.iImage = iidx;
	ListView_SetItem(hwnd_list, &lvi);
}


/*--------------------------------------------------------------------------
 
  get_list_param

  XgACelParam擾

--------------------------------------------------------------------------*/

LPARAM get_list_param(int item)
{
	LVITEM lvi;

	lvi.iItem = item;
	lvi.iSubItem = 0;
	lvi.mask = TVIF_PARAM;
	if (ListView_GetItem(hwnd_list, &lvi))
		return lvi.lParam;
	return -1;
}


/*--------------------------------------------------------------------------
 
  list_compare

  Xgr[\[gpR[obN֐

--------------------------------------------------------------------------*/

int CALLBACK list_compare(LPARAM lp1, LPARAM lp2, LPARAM lp3)
{
	GSureList* surelist;
	int sortkey = HIWORD(lp3);
	int sortdir = LOWORD(lp3);
	int type = HIWORD(lp1);
	int i1 = LOWORD(lp1);
	int i2 = LOWORD(lp2);
	int i3;

	if (sortdir == 2)
	{
		i3 = i1;
		i1 = i2;
		i2 = i3;
	}

	switch (list_type)
	{
	case 0: surelist = surelist_def; break;
	case 1: surelist = surelist_fav; break;
	case 2: surelist = surelist_rec; break;
	}
    
	switch (sortkey)
    {
    case 0:
        return (i1 - i2);
        break;
    case 1:
		return strcmp(surelist->get_title(i1), surelist->get_title(i2));
        break;
    case 2:
		return (surelist->get_res(i1) - surelist->get_res(i2));
        break;
    case 3:
		return strcmp(surelist->get_from(i1), surelist->get_from(i2));
        break;
    }

    return 0;   
}


/*--------------------------------------------------------------------------
 
  threadentry_check_thread

  XXV`FbNXbh

--------------------------------------------------------------------------*/

unsigned int WINAPI threadentry_check_thread(void* data)
{
	GSure td;
	LPARAM param;
	char url[256];
	char _url[256];
	bool end_flag;
	int idx;
	int st;
	UINT i;

	if (timer_id == 0)
		timer_id = SetTimer(hwnd_main, IDT_LAMPBLINK, 1000, NULL);
	
	do
	{
		i = 0;
		while (i < (unsigned)ListView_GetItemCount(hwnd_list))
		{
			param = get_list_param(i);
			idx = LOWORD(param);
			if (surelist_fav->get_flag(idx) != 0 &&
				surelist_fav->get_flag(idx) != 3 &&
				surelist_fav->get_type(idx) != 255)
			{
			    surelist_fav->get_url(idx, 0, _url);
				surelist_fav->set_flag(idx, 2);
				if (list_type == 1)
					set_list_icon(i);

				td.get(_url, log_dir, &st, false);
				if (st > 0)
				{
					surelist_fav->set_flag(idx, 3);
					switch (surelist_fav->get_type(idx))
					{
					case 0: wsprintf(url, "%s&st=%d&to=%d", _url, st, st);
					case 1: wsprintf(url, "%s&START=%d", _url, st);
					}
					if (tab_automove_check)
						open_tab(url, surelist_fav->get_type(idx), surelist_fav->get_title(idx));
				}
				else
				{
					surelist_fav->set_flag(idx, 0);
				}
				if (list_type == 1)
					set_list_icon(i);
			}
			i++;
		}

		end_flag = true;
		for (i = 0; i < surelist_fav->count(); i++)
		{
			if (surelist_fav->get_flag(i) != 0 && surelist_fav->get_flag(i) != 3)
			{
				end_flag = false;
				break;
			}
		}
	}
	while (!end_flag);

	CloseHandle(handle_check_thread);
	handle_check_thread = NULL;
	_endthreadex(0);
	return 0;
}


/*--------------------------------------------------------------------------
 
  dlg_sureprop_Proc

  XEvpeB_CAOEvV[W

--------------------------------------------------------------------------*/

BOOL CALLBACK dlg_sureprop_Proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	static GSureList* surelist;
	static int idx;
	char title[256];
	char from[MAX_FROM_NAME];
	char logfile[MAX_PATH];
	LPARAM param;

	switch (msg)
	{
	case WM_INITDIALOG:
		param = get_list_param(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED));
		idx = LOWORD(param);
		switch (list_type)
		{
		case 0: surelist = surelist_def; break;
		case 1: surelist = surelist_fav; break;
		case 2: surelist = surelist_rec; break;
		}
		dlg_sureprop_OnInit(hwnd, surelist, idx);
		set_window_pos_best(hwnd);
		break;

	case WM_COMMAND:
		switch (LOWORD(wp))
		{
		case IDC_TO_OFFLINE:
			if (MessageBox(hwnd, "Offline֕ύXƁAJƂɍXV擾ȂȂ܂B\n낵łH", "DAT֕ύX", MB_OKCANCEL | MB_ICONQUESTION) == IDOK)
			{
				Edit_GetText(GetDlgItem(hwnd, IDC_TITLE), title, sizeof(title));
				Edit_GetText(GetDlgItem(hwnd, IDC_FROM),  from,  sizeof(from));
				surelist->get_log_filename(logfile, NULL, idx);
				surelist->del(idx);
				surelist->insert(idx, logfile, "", "", title, 0, 255, from);
				dlg_sureprop_OnInit(hwnd, surelist, idx);
			}
			break;
		//OK
		case IDOK:
			Edit_GetText(GetDlgItem(hwnd, IDC_TITLE), title, sizeof(title));
			Edit_GetText(GetDlgItem(hwnd, IDC_FROM),  from,  sizeof(from));
			surelist->modify(idx, title, from);
			save_thread();
			
		//CANCEL
		case IDCANCEL:
			EndDialog(hwnd,  LOWORD(wp));
			break;
		}
		break;

	default:
		return FALSE;
	}
	return TRUE;
}


/*--------------------------------------------------------------------------
 
  dlg_sureprop_OnInit

  XEvpeB_CAO

--------------------------------------------------------------------------*/

void dlg_sureprop_OnInit(HWND hwnd, GSureList* surelist, UINT idx)
{
	Edit_LimitText(GetDlgItem(hwnd, IDC_TITLE), 256);
	Edit_LimitText(GetDlgItem(hwnd, IDC_FROM),  64);
	
	if (surelist->get_type(idx) != 255)
	{
		Edit_SetText(GetDlgItem(hwnd, IDC_TITLE),  surelist->get_title(idx));
		Edit_SetText(GetDlgItem(hwnd, IDC_FROM),   surelist->get_from(idx));
		Edit_SetText(GetDlgItem(hwnd, IDC_SERVER), surelist->get_server(idx));
		Edit_SetText(GetDlgItem(hwnd, IDC_BBS),    surelist->get_bbs(idx));
		Edit_SetText(GetDlgItem(hwnd, IDC_KEY),    surelist->get_key(idx));
		Edit_SetText(GetDlgItem(hwnd, IDC_TYPE),   "Online");
	}
	else
	{
		Edit_SetText(GetDlgItem(hwnd, IDC_TITLE),  surelist->get_title(idx));
		Edit_SetText(GetDlgItem(hwnd, IDC_FROM),   surelist->get_from(idx));
		Edit_SetText(GetDlgItem(hwnd, IDC_SERVER), "-");
		Edit_SetText(GetDlgItem(hwnd, IDC_BBS),    "-");
		Edit_SetText(GetDlgItem(hwnd, IDC_KEY),    "-");
		Edit_SetText(GetDlgItem(hwnd, IDC_TYPE),   "Offline");
		EnableWindow(GetDlgItem(hwnd, IDC_TO_DAT), FALSE);
	}
}


/*--------------------------------------------------------------------------
 
  import_dat

  DAT̃C|[g

--------------------------------------------------------------------------*/

bool import_dat(void)
{
    OPENFILENAME ofn;
	FILE *in;
	FILE *out;
	char infile[MAX_PATH];
    char intitle[MAX_PATH];
	char outfile[MAX_PATH];
	char title[256];
	char* buff;
	char* p1;
	char* p2;
	char* p3;
	char c;
	int maxlen;
	int len;
	int cnt;
	struct tm tm_wtime;
	time_t t;
	int i;

	//t@CI
	strcpy(infile, "");
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwnd_list;
	ofn.lpstrFilter = "DATt@C (*.dat;*.txt)\0*.dat;*.txt\0ׂẴt@C (*.*)\0*.*\0\0";
	ofn.lpstrFile = infile;
	ofn.nMaxFile = MAX_PATH;
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.lpstrDefExt = "dat";
	ofn.nMaxFileTitle = MAX_PATH;
	ofn.lpstrFileTitle = intitle;
	ofn.lpstrTitle = "DATt@C̃C|[g";

	if (!GetOpenFileName(&ofn))
	{
		return true;
	}
	wsprintf(outfile, "%s\\%s", log_dir, intitle);


	//ϊ
	change_to_moduledir();
	if ((in = fopen(infile, "r")) == NULL)
	{
		return false;
	}
	if ((out = fopen(outfile, "w")) == NULL)
	{
		fclose(in);
		return false;
	}

	//get maxlen
	maxlen = 0;
	len = 0;
	while ((c = fgetc(in)) != EOF)
	{
		len++;
		if (c == '\n')
		{
			if (maxlen < len)
				maxlen = len;
			len = 0;
		}
	}
	maxlen++;
	buff = new char[maxlen];
	rewind(in);

	//get title
	if (fgets(buff, maxlen, in) == NULL)
	{
		delete [] buff;
		return false;
	}
	p1 = buff;
	for (i = 0; i < 4; i++)
	{
		if ((p2 = strstr(p1, "<>")) == NULL)
		{
			fclose(in);
			fclose(out);
			delete [] buff;
			return false;
		}
		p1 = p2 + 2;
	}
	*(p1 + strlen(p1) - 1) = '\0'; //s
	strcpy(title, p1);
	fprintf(out, "%s\n", p1);
	rewind(in);

	//get line
	cnt = 0;
	while (fgets(buff, maxlen, in) != NULL)
	{
		//name
		p1 = buff;
		if ((p2 = strstr(p1, "<>")) == NULL) continue;
		*p2 = '\0';
		fprintf(out, "%d\t%s", cnt, p1);
		p1 = p2 + 2;

		//mail
		if ((p2 = strstr(p1, "<>")) == NULL) continue;
		*p2 = '\0';
		fprintf(out, "\t%s", p1);
		p1 = p2 + 2;

		//time
		if ((p2 = strstr(p1, "<>")) == NULL) continue;
		*p2 = '\0';
		if ((p3 = strstr(p1, "ID:")) != NULL)
			p3 += 3;
		t = 0;
		memset(&tm_wtime, 0, sizeof(tm_wtime));
		p1 = strtok(p1, " /:()");
		if (strlen(p1) == 2)
			tm_wtime.tm_year = atoi(p1) + 100;
		else
			tm_wtime.tm_year = atoi(p1) - 1900;
		p1 = strtok(NULL, " /:()");
		tm_wtime.tm_mon = atoi(p1) - 1;
		p1 = strtok(NULL, " /:()");
		tm_wtime.tm_mday = atoi(p1);
		p1 = strtok(NULL, " /:()");
		if (strspn(p1, "0123456789") == 0)
		p1 = strtok(NULL, " /:()");
		tm_wtime.tm_hour = atoi(p1);
		p1 = strtok(NULL, " /:()");
		tm_wtime.tm_min = atoi(p1);
		t = mktime(&tm_wtime);
		fprintf(out, "\t%ld", t);

		//id
		if (p3 != NULL)
			fprintf(out, "\t%s", p3);
		else
			fprintf(out, "\t", p3);
		p1 = p2 + 2;
		
		//text
		if ((p2 = strstr(p1, "<>")) == NULL) continue;
		*p2 = '\0';
		while ((p2 = strstr(p1, "<br>")) != NULL)
		{
			*p2 = '\0';
			fprintf(out, "\t%s",  p1);
			p1 = p2 + 4;
		}
		fprintf(out, "\t%s\t\n",  p1);
		cnt++;
	}

	fclose(in);
	fclose(out);
	delete [] buff;

	//Cɓɒǉ
	surelist_fav->insert(surelist_fav->count(), intitle, "", "", title, 0, 255, "");
	save_thread();
	if (list_type == 1)
		make_surelist(1);
	
	return true;
}

/*--------------------------------------------------------------------------
 
  import_dat

  DAT̃C|[g

--------------------------------------------------------------------------*/

bool export_dat(void)
{
    OPENFILENAME ofn;
	ExportDat exp;
	char infile[MAX_PATH];
	char outfile[MAX_PATH];
    char outtitle[MAX_PATH];
	LPARAM idx;

	idx = LOWORD(get_list_param(ListView_GetNextItem(hwnd_list, -1, LVIS_SELECTED)));

	//t@CI
	surelist_fav->get_log_filename(outfile, NULL, idx);
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwnd_list;
	ofn.lpstrFilter = "DATt@C (*.dat)\0*.dat\0ׂẴt@C (*.*)\0*.*\0\0";
	ofn.lpstrFile = outfile;
	ofn.nMaxFile = MAX_PATH;
	ofn.Flags = OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt = "dat";
	ofn.nMaxFileTitle = MAX_PATH;
	ofn.lpstrFileTitle = outtitle;
	ofn.lpstrTitle = "DATt@C֏o";

	if (!GetSaveFileName(&ofn))
	{
		return true;
	}

	change_to_moduledir();
	surelist_fav->get_log_filename(infile, log_dir, idx);

	return exp.output(infile, outfile);
}
