/***************************************************************************
  Tab.cpp

  Copyright (C) 2001 Hachimaki. All rights reserved.
  Licensed under the GPL(v2).
***************************************************************************/

#include "GBCommon.h"
#include "GickoViewer.h"
#include <process.h>

//OQ
extern HINSTANCE instance;
extern HWND hwnd_main;
extern HWND hwnd_tab;
extern HWND hwnd_tree;
extern HWND hwnd_list;
extern HWND hwnd_edit;
extern HFONT tabfont;
extern HANDLE handle_open_url;
extern UINT timer_id;
extern GViewer* viewer;
extern char log_dir[MAX_PATH];
extern int tab_height;
extern int tab_max_strlen;
extern BOOL tab_automove_sure;

//O[oϐ
static FARPROC org_tab_proc;
static vector<TabParam*> open_url_queue;

//֐vg^Cv
static LRESULT CALLBACK tab_proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp);
static void set_tab_image(int idx, int iidx);
static int  find_tab(TabParam* param);
static UINT WINAPI threadentry_open_url(void* param);


/*--------------------------------------------------------------------------
 
  create_tab

  ^uRg[쐬

--------------------------------------------------------------------------*/

void create_tab(void)
{
	HIMAGELIST imagelist;
	NONCLIENTMETRICS ncm;
	LOGFONT lftab;
	HDC hdc;
	HFONT oldfont;
	SIZE size;
	TCITEM tci;

	hwnd_tab = CreateWindowEx(
        0,
        WC_TABCONTROL,
        "",
        WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
		TCS_FOCUSNEVER /*| TCS_BUTTONS | TCS_FLATBUTTONS*/ /*| TCS_BOTTOM | TCS_OWNERDRAWFIXED*/,
        0, 0, 0, 0,
        hwnd_main,
        (HMENU)ID_TAB,
        instance,
        NULL);

	//C[WXg
	imagelist = ImageList_LoadBitmap(instance, MAKEINTRESOURCE(IDB_TABIMAGELIST), 5, 0, CLR_NONE);
	TabCtrl_SetImageList(hwnd_tab, imagelist);

	//tHgݒ
	ncm.cbSize = sizeof(ncm);
	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(ncm), &ncm, 0);
	memcpy(&lftab, &ncm.lfMenuFont, sizeof(lftab));
	tabfont = CreateFontIndirect(&lftab);
    SendMessage(hwnd_tab, WM_SETFONT, (WPARAM)tabfont, MAKELPARAM(FALSE, 0));

	//^uACeTCYݒ
	hdc = GetDC(hwnd_tab);
	oldfont = (HFONT)SelectObject(hdc, tabfont);
	GetTextExtentPoint32(hdc, "A", 1, &size);
	SelectObject(hdc, oldfont);
	ReleaseDC(hwnd_tab, hdc);
	tab_height = size.cy + 5;
	TabCtrl_SetItemSize(hwnd_tab, -1, tab_height);

    //{^uACeǉ
    tci.mask = TCIF_TEXT;
    tci.pszText = "";
    TabCtrl_InsertItem(hwnd_tab, 0, &tci);

    tci.pszText = "X";
    TabCtrl_InsertItem(hwnd_tab, 1, &tci);

    //TuNX
	org_tab_proc = (FARPROC)GetWindowLong(hwnd_tab, GWL_WNDPROC);
	SetWindowLong(hwnd_tab, GWL_WNDPROC, (LONG)tab_proc);
}


/*----------------------------------------------------------------------------

  tab_OnInetCallback

  ^uɑ΂UM_INET_CALLBACK̏

----------------------------------------------------------------------------*/

void tab_OnInetCallback(UINT message_id, LPARAM lp)
{
	TabParam* param;
	char url[256];
	int idx;

	param = (TabParam*)lp;

	switch (message_id)
	{
	//ڑ
	case 0:
		if ((idx = find_tab(param)) >= 2)
			set_tab_image(idx, 0);
		break;

	//M
	case 1:
		if ((idx = find_tab(param)) >= 2)
			set_tab_image(idx, 1);
		break;

	//N[Y
	case 2:
		if ((idx = find_tab(param)) >= 2)
			set_tab_image(idx, -1);
		break;

	//
	case 4:
		if ((idx = find_tab(param)) >= 2)
		{
			if (idx == TabCtrl_GetCurSel(hwnd_tab))
			{
				param->make(url);
				viewer->set_cgiver(param->cgiver);
				viewer->open(url, log_dir, (param->type == 255 ? true : false), param->top_line);
			}
		}
		break;
	}
}


/*--------------------------------------------------------------------------

  tab_OnContextMenu

  ENbNj[\

--------------------------------------------------------------------------*/

void tab_OnContextMenu(WORD x, WORD y)
{
	TCHITTESTINFO tab_ht;
	MENUITEMINFO mii;
	HMENU menu1;
	HMENU menu2;
	int idx;

	tab_ht.pt.x = x;
	tab_ht.pt.y = y;
	ScreenToClient(hwnd_tab, &tab_ht.pt);
	
	if ((idx = TabCtrl_HitTest(hwnd_tab, &tab_ht)) < 0)
		return;
	
	switch (idx)
	{
	case 0:
		break;
	case 1:
		menu1 = LoadMenu(instance, MAKEINTRESOURCE(IDR_TTAB_POPUP));
		menu2 = GetSubMenu(menu1, 0);
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_STATE;
		mii.fState = MFS_CHECKED;
		SetMenuItemInfo(menu2, get_list_type(), TRUE, &mii);
		TrackPopupMenu(menu2, TPM_LEFTALIGN | TPM_LEFTBUTTON, x, y, 0, hwnd_main, NULL);
		DestroyMenu(menu1);
		break;
	default:
		del_tab(idx);
		break;
	}
}


/*----------------------------------------------------------------------------

  tab_OnSelChange

  WM_NOTIFY/TCN_SELCHANGE ̏

----------------------------------------------------------------------------*/

void tab_OnSelChange(void)
{
	TabParam* param;
	char title[256];
	char url[256];
	int cursel;

	cursel = SendMessage(hwnd_tab, TCM_GETCURSEL, 0, 0);

	//^u
	if (cursel == 0)
	{
		ShowWindow(hwnd_tree, SW_SHOWNORMAL);
		ShowWindow(hwnd_list, SW_HIDE);
		ShowWindow(hwnd_edit, SW_HIDE);
		viewer->show(false);
		SetFocus(hwnd_tree);

		SetWindowText(hwnd_main, WINDOW_TITLE);
	}
	//X^u
	else if (cursel == 1)
	{
		ShowWindow(hwnd_tree, SW_HIDE);
		ShowWindow(hwnd_list, SW_SHOWNORMAL);
		ShowWindow(hwnd_edit, SW_SHOWNORMAL);
		viewer->show(false);
		SetFocus(hwnd_list);

		//wsprintf(title, "%s - %s", italist_2ch->get_name(bindex), WINDOW_TITLE);
		SetWindowText(hwnd_main, WINDOW_TITLE);
	}
	//Viewer^u
	else
	{
		ShowWindow(hwnd_tree, SW_HIDE);
		ShowWindow(hwnd_list, SW_HIDE);
		ShowWindow(hwnd_edit, SW_HIDE);
		viewer->show(true);

		if ((param = get_tab_param(cursel)) != NULL)
		{
			if (param->flag == 0)
			{
				if (param->type != 255)
				{
					param->make(url);
					viewer->set_cgiver(param->cgiver);
					viewer->open(url, log_dir, false, param->top_line);
				}
				else
				{
					viewer->open(param->url, log_dir, true, param->top_line);
				}
			}
			else
			{
				viewer->open("", log_dir, false, 0);
			}
			SetFocus(viewer->get_handle());

			wsprintf(title, "%s - %s", param->title, WINDOW_TITLE);
			SetWindowText(hwnd_main, title);
		}
	}
	set_control_pos();
}


/*--------------------------------------------------------------------------

  select_tab

  ^uI

--------------------------------------------------------------------------*/

void select_tab(int idx)
{
	if (idx >= 0 &&
		idx < TabCtrl_GetItemCount(hwnd_tab) &&
		idx != TabCtrl_GetCurSel(hwnd_tab))
	{
		TabCtrl_SetCurFocus(hwnd_tab, idx);
	}
}


/*--------------------------------------------------------------------------

  select_next_tab

  ̃^uI

--------------------------------------------------------------------------*/

void select_next_tab(void)
{
	TabCtrl_SetCurFocus(
		hwnd_tab,
		(TabCtrl_GetCurSel(hwnd_tab) + 1) %
		TabCtrl_GetItemCount(hwnd_tab));
}


/*--------------------------------------------------------------------------

  select_prev_tab

  Õ^uI

--------------------------------------------------------------------------*/

void select_prev_tab(void)
{
	TabCtrl_SetCurFocus(
		hwnd_tab,
		(TabCtrl_GetCurSel(hwnd_tab) - 1 + TabCtrl_GetItemCount(hwnd_tab)) %
		TabCtrl_GetItemCount(hwnd_tab));
}


/*--------------------------------------------------------------------------

  open_tab

  wURL̃^u}HTMLǂݍ݃Xbhs

--------------------------------------------------------------------------*/

void open_tab(const char* url, UCHAR type, const char* title)
{
	GSure sure;
	TabParam* new_param;
	TabParam* param;
	char cmdline[256];
	bool found;
	unsigned int taddr;
	int i;

	new_param = new TabParam();
	strcpy(new_param->url, url);

	//URL`FbN
	if (type != 255)
	{
		if (!new_param->split(new_param->url))
		{
			//2chURLȊO̓uEUɓn
			delete new_param;
			wsprintf(cmdline, "exec.exe %s", url);
			WinExec(cmdline, SW_SHOWNORMAL);
			return;
		}
	}
	else
	{
		strcpy(new_param->url, url);
		new_param->type = 255;
	}

	//Ƀ^u邩`FbN
	found = false;
	for (i = 2; i < TabCtrl_GetItemCount(hwnd_tab); i++)
	{
		param = get_tab_param(i);
		if (strcmp(param->server, new_param->server) == 0 &&
			strcmp(param->bbs,    new_param->bbs)    == 0 &&
			strcmp(param->key,    new_param->key)    == 0)
		{
			found = true;
			break;
		}
	}

	if (found)
	{
		//^u
		delete new_param;
		new_param = param;
		strcpy(new_param->url, url);
	}
	else
	{
		//^u
		TCITEM tci;
		tci.mask = TCIF_PARAM;
		tci.lParam = (LPARAM)new_param;
		i = TabCtrl_InsertItem(hwnd_tab, INT_MAX, &tci);

		if (title == NULL || strlen(title) == 0)
			set_tab_text(i, new_param->url);
		else
			set_tab_text(i, title);
	}

	//L[ɓ
	if (type != 255)
	{
		if (new_param->flag == TABFLAG_NONE)
		{
			new_param->flag = TABFLAG_WAIT;
			open_url_queue.push_back(new_param);
			set_tab_image(find_tab(new_param), 0);
		}

		//XbhJn
		if (handle_open_url == NULL)
			handle_open_url = (HANDLE)_beginthreadex(NULL, 0,
 										  threadentry_open_url, 0, 0, &taddr);
	}

	//^uI
	if (tab_automove_sure)
		select_tab(find_tab(new_param));
}


/*--------------------------------------------------------------------------
 
  del_tab

  ^u폜

--------------------------------------------------------------------------*/

void del_tab(int idx)
{
	TabParam* param;

	if (idx < 2)
		return;

	if ((param = get_tab_param(idx)) != NULL)
		delete param;

	TabCtrl_DeleteItem(hwnd_tab, idx);

	if (TabCtrl_GetCurSel(hwnd_tab) == -1)
	{
		if (idx < TabCtrl_GetItemCount(hwnd_tab))
			select_tab(idx);
		else
			select_tab(idx - 1);
	}
}


/*--------------------------------------------------------------------------

  set_tab_text

  ^uRg[̃eLXgZbg

--------------------------------------------------------------------------*/

void set_tab_text(int i, const char* _title)
{
	TabParam* param;
	char title[256];
	TCITEM tci;

	if ((param = get_tab_param(i)) != NULL)
	{
		strcpy(param->title, _title);
	}

	strcpy(title, _title);
	if ((signed)strlen(title) > tab_max_strlen)
	{
		if (get_bytetype(title, tab_max_strlen) != 2)
			*(title + tab_max_strlen) = '\0';
		else
			*(title + tab_max_strlen - 1) = '\0';
	}

	tci.mask = TCIF_TEXT;
	tci.pszText = title;
	tci.cchTextMax = strlen(title);
	TabCtrl_SetItem(hwnd_tab, i, &tci);
}


/*--------------------------------------------------------------------------

  tab_proc

  ^ũvV[W

--------------------------------------------------------------------------*/

static LRESULT CALLBACK tab_proc(HWND hwnd, UINT msg, WPARAM wp, LPARAM lp)
{
	static int dragtab = -1;
	TCHITTESTINFO hti;
	TCITEM tci;
	char title[256];
	int i;

    switch (msg)
	{
	case WM_LBUTTONDOWN:
		hti.pt.x = LOWORD(lp);
		hti.pt.y = HIWORD(lp);
		i = TabCtrl_HitTest(hwnd, &hti);
		if (i >= 2)
		{
			SetCapture(hwnd);
			dragtab = i;
		}
		break;
	case WM_MOUSEMOVE:
		if (dragtab != -1)
		{
			hti.pt.x = LOWORD(lp);
			hti.pt.y = HIWORD(lp);
			i = TabCtrl_HitTest(hwnd, &hti);
			if (i >= 2 && (i == dragtab + 1 || i == dragtab - 1))
			{
				tci.mask = TCIF_TEXT | TCIF_IMAGE | TCIF_PARAM;
				tci.pszText = title;
				tci.cchTextMax = sizeof(title);
				TabCtrl_GetItem(hwnd, dragtab, &tci);
				TabCtrl_DeleteItem(hwnd, dragtab);
				TabCtrl_InsertItem(hwnd, i, &tci);
				dragtab = i;
			}
		}
		break;
	case WM_LBUTTONUP:
		if (dragtab != -1)
		{
			ReleaseCapture();
			dragtab = -1;
		}
		break;
    }
    return (CallWindowProc((WNDPROC)org_tab_proc, hwnd, msg, wp, lp));
}


/*--------------------------------------------------------------------------

  set_tab_image

  ^ũC[WZbg

--------------------------------------------------------------------------*/

void set_tab_image(int idx, int iidx)
{
	TCITEM tci;

	tci.mask = TCIF_IMAGE;
	tci.iImage = iidx;
	TabCtrl_SetItem(hwnd_tab, idx, &tci);
}


/*--------------------------------------------------------------------------

  find_tab

  wlParam̃^uT

--------------------------------------------------------------------------*/

int find_tab(TabParam* param)
{
	TCITEM tci;
	int i;

	for (i = 2; i < TabCtrl_GetItemCount(hwnd_tab); i++)
	{
		tci.mask = TCIF_PARAM;
		TabCtrl_GetItem(hwnd_tab, i, &tci);
		if ((TabParam*)tci.lParam == param)
			return i;
	}
	return -1;
}


/*--------------------------------------------------------------------------

  get_tab_param

  ^ulParam擾

--------------------------------------------------------------------------*/

TabParam* get_tab_param(int idx)
{
	TCITEM tci;

	if (idx < 2)
		return NULL;
	
	tci.mask = TCIF_PARAM;
	TabCtrl_GetItem(hwnd_tab, idx, &tci);
	return (TabParam*)tci.lParam;
}


/*--------------------------------------------------------------------------
 
  threadentry_open_url

  URLI[vXbh

--------------------------------------------------------------------------*/

UINT WINAPI threadentry_open_url(void* param)
{
	TabParam* open_tab_param;
	GSure sure;
	int idx;
	int st;

	if (timer_id == 0)
		timer_id = SetTimer(hwnd_main, IDT_LAMPBLINK, 1000, NULL);

	while (open_url_queue.size() > 0)
	{
		open_tab_param = open_url_queue[0];
		open_url_queue.erase(&open_url_queue[0]);
		if ((idx = find_tab(open_tab_param)) != -1)
		{
			open_tab_param->flag = TABFLAG_LOADING;
			sure.set_callback(hwnd_main, UM_INET_CALLBACK, (LPARAM)open_tab_param);
			sure.get(open_tab_param->url, log_dir, &st, false);

			if ((idx = find_tab(open_tab_param)) == -1) //[hɍ폜ꂽ
				continue;
			set_tab_text(idx, sure.get_title());
			strcpy(open_tab_param->cgiver, sure.get_cgiver());
			PostMessage(hwnd_main, UM_INET_CALLBACK, 4, (LPARAM)open_tab_param);
			open_tab_param->flag = TABFLAG_NONE;
		}
	}

	CloseHandle(handle_open_url);
	handle_open_url = NULL;
	_endthreadex(0);
	return 0;
}
